n=grp2idx(imds.Labels);
%only_saliency_mat=[saliency_dense_features,n];

%% Fine Tree
fineTree = fitctree(...
    saliency_dense_features, ...
    n, ...
    'SplitCriterion', 'gdi', ...
    'MaxNumSplits', 100, ...
    'Surrogate', 'off', ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(fineTree, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('FineTree ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')

%% medium tree
mediumTree = fitctree(...
    saliency_dense_features, ...
    n, ...
    'SplitCriterion', 'gdi', ...
    'MaxNumSplits', 20, ...
    'Surrogate', 'off', ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(mediumTree, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('MediumTree Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Coarse tree
coraseTree = fitctree(...
    saliency_dense_features, ...
    n, ...
    'SplitCriterion', 'gdi', ...
    'MaxNumSplits', 4, ...
    'Surrogate', 'off', ...
    'ClassNames', [single(1); single(2)]);

% Perform cross-validation
partitionedModel = crossval(coraseTree, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('CoarseTree Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Linear discriminant
linearDiscriminant = fitcdiscr(...
    saliency_dense_features, ...
    n, ...
    'DiscrimType', 'linear', ...
    'Gamma', 0, ...
    'FillCoeffs', 'off', ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(linearDiscriminant, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Linear discriminant Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% L-SVM
LSVM = fitcsvm(...
    saliency_dense_features, ...
    n, ...
    'KernelFunction', 'linear', ...
    'PolynomialOrder', [], ...
    'KernelScale', 'auto', ...
    'BoxConstraint', 1, ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(LSVM, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('L-SVM Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Q-SVM
QSVM = fitcsvm(...
    saliency_dense_features, ...
    n, ...
    'KernelFunction', 'polynomial', ...
    'PolynomialOrder', 2, ...
    'KernelScale', 'auto', ...
    'BoxConstraint', 1, ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);

% Perform cross-validation
partitionedModel = crossval(QSVM, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Q-SVM Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Cubic-SVM
CubicSVM = fitcsvm(...
    saliency_dense_features, ...
    n, ...
    'KernelFunction', 'polynomial', ...
    'PolynomialOrder', 3, ...
    'KernelScale', 'auto', ...
    'BoxConstraint', 1, ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);

% Perform cross-validation
partitionedModel = crossval(CubicSVM, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('CubicSVM Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% fine-gussian SVM
FGSVM = fitcsvm(...
    saliency_dense_features, ...
    n, ...
    'KernelFunction', 'gaussian', ...
    'PolynomialOrder', [], ...
    'KernelScale', 2.5, ...
    'BoxConstraint', 1, ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(FGSVM, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Fine-Gaussian SVM Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Medium Guassian SVM
MGSVM = fitcsvm(...
    saliency_dense_features, ...
    n, ...
    'KernelFunction', 'gaussian', ...
    'PolynomialOrder', [], ...
    'KernelScale', 10, ...
    'BoxConstraint', 1, ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(MGSVM, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Medium Gussian SVM Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Coarse guasian SVM
CGSVM = fitcsvm(...
    saliency_dense_features, ...
    n, ...
    'KernelFunction', 'gaussian', ...
    'PolynomialOrder', [], ...
    'KernelScale', 41, ...
    'BoxConstraint', 1, ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(CGSVM, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Coarse Gussian SVM Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% fine-KNN
fineKNN = fitcknn(...
    saliency_dense_features, ...
    n, ...
    'Distance', 'Euclidean', ...
    'Exponent', [], ...
    'NumNeighbors', 1, ...
    'DistanceWeight', 'Equal', ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(fineKNN, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Fine-KNN Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Medium KNN
medKNN = fitcknn(...
    saliency_dense_features, ...
    n, ...
    'Distance', 'Euclidean', ...
    'Exponent', [], ...
    'NumNeighbors', 10, ...
    'DistanceWeight', 'Equal', ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(medKNN, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Medium KNN Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Coarse KNN
CoarseKNN = fitcknn(...
    saliency_dense_features, ...
    n, ...
    'Distance', 'Euclidean', ...
    'Exponent', [], ...
    'NumNeighbors', 100, ...
    'DistanceWeight', 'Equal', ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(CoarseKNN, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Coarse-KNN Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Cosine-KNN
CosineKNN = fitcknn(...
    saliency_dense_features, ...
    n, ...
    'Distance', 'Cosine', ...
    'Exponent', [], ...
    'NumNeighbors', 10, ...
    'DistanceWeight', 'Equal', ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);

% Perform cross-validation
partitionedModel = crossval(CosineKNN, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Cosine-KNN Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Cubic-KNN
cubicKNN = fitcknn(...
    saliency_dense_features, ...
    n, ...
    'Distance', 'Minkowski', ...
    'Exponent', 3, ...
    'NumNeighbors', 10, ...
    'DistanceWeight', 'Equal', ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(cubicKNN, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Cubic-KNN Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Weighted KNN
weightedKNN = fitcknn(...
    saliency_dense_features, ...
    n, ...
    'Distance', 'Euclidean', ...
    'Exponent', [], ...
    'NumNeighbors', 10, ...
    'DistanceWeight', 'SquaredInverse', ...
    'Standardize', true, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(weightedKNN, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Weighted-KNN Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Enesmble boost tree
template = templateTree(...
    'MaxNumSplits', 20);
boostEnsemble = fitcensemble(...
    saliency_dense_features, ...
    n, ...
    'Method', 'AdaBoostM1', ...
    'NumLearningCycles', 30, ...
    'Learners', template, ...
    'LearnRate', 0.1, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(boostEnsemble, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Enesmble boost Tree Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')
%% Enesmbe bag tree
template = templateTree(...
    'MaxNumSplits', 646);
bagEnsemble = fitcensemble(...
    saliency_dense_features, ...
    n, ...
    'Method', 'Bag', ...
    'NumLearningCycles', 30, ...
    'Learners', template, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(bagEnsemble, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Enesmble Bag Tree Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')


%% rus boost tree
template = templateTree(...
    'MaxNumSplits', 20);
rusboostEnsemble = fitcensemble(...
    saliency_dense_features, ...
    n, ...
    'Method', 'RUSBoost', ...
    'NumLearningCycles', 30, ...
    'Learners', template, ...
    'LearnRate', 0.1, ...
    'ClassNames', [single(1); single(2)]);
% Perform cross-validation
partitionedModel = crossval(rusboostEnsemble, 'KFold', 5);

% Compute validation predictions
[validationPredictions, validationScores] = kfoldPredict(partitionedModel);

% Compute validation accuracy
disp('Enesmble RUSBoost Tree Accuracy: ')
validationAccuracy = 1 - kfoldLoss(partitionedModel, 'LossFun', 'ClassifError')