imageFolder = fullfile('C:\Users\Talha\Downloads\Compressed\Dataset_BUSI\Dataset_BUSI_with_GT\','segmented_masks');

imds = imageDatastore(imageFolder, ...
    'IncludeSubfolders',true, ...
    'LabelSource','foldernames');
%%
[trainingSet, testSet] = splitEachLabel(imds, 0.7, 'randomize');
%%

for i=1:length(trainingSet.Labels)
    I=readimage(trainingSet,i);
    [regions,mserCC] = detectMSERFeatures(I);
%Show all detected MSER Regions.

    figure
    imshow(I)
    hold on
    plot(regions,'showPixelList',true,'showEllipses',false)
    T_Hcorners = detectHarrisFeatures(I);
    [T_Hfeatures, valid_corners] = extractFeatures(I, T_Hcorners);
    if(isempty(T_Hfeatures.Features))
        T_f1(i,:)=0;
    else
    T_f1(i,1:30)=T_Hfeatures.Features(1:1,1:30);
    end
    T_Spoints = detectSURFFeatures(I);
    [T_Sfeatures, valid_points] = extractFeatures(I, T_Spoints);
    if(isempty(T_Sfeatures))
        T_f1(i,:)=0;
    else
    T_f2(i,1:30)=T_Sfeatures(1:1,1:30);
    end
end
 %%
for i=1:length(testSet.Labels)
    I=readimage(testSet,i);
    V_Hcorners = detectHarrisFeatures(I);
   [V_Hfeatures, valid_corners] = extractFeatures(I, V_Hcorners);
    if(isempty(V_Hfeatures.Features))
        V_f1(i,:)=0;
    else
    V1_f1(i,1:30)=V_Hfeatures.Features(1:1,1:30);
    end
    V_Spoints = detectSURFFeatures(I);
    [V_Sfeatures, valid_points] = extractFeatures(I, V_Spoints);
     if(isempty(V_Sfeatures))
        V_f2(i,:)=0;
    else
    V_f2(i,1:30)=V_Sfeatures(1:1,1:30);
    end
end
%%
training_features=[T_f1,T_f2];
testing_features=[V1_f1,V_f2];
%%
net=fitcecoc(training_features,trainingSet.Labels);
YPred = predict(net,testing_features);
YTest = testSet.Labels;
%Calculate the accuracy. The accuracy is the ratio of the number of true labels in the test data matching the classifications from classify to the number of images in the test data.

accuracy = sum(YPred == YTest)/numel(YTest)
%%
%length(imds.Labels)
for i=1:length(imds.Labels)
    I=readimage(imds,i);
    [Saliency_Map, Feature_Maps, ICA_Maps, img, SUN] = Run_SUN(I);
    saliency_features{i}=mean(Feature_Maps, 3);
end
%%
for i=1:length(saliency_features)
    saliency_features_mean(i,:)=mean(saliency_features{i});
end
%%
img = benign_3_;figure;
          subplot(1, 4, 1);imagesc(img);title('Original Image');axis off;
          [Saliency_Map, Feature_Maps, ICA_Maps, img] = ...
              Run_SUN(img, []);
          subplot(1, 4, 2);imagesc(img);title('Preprocessed Image');
          axis off;
          subplot(1, 4, 3);imagesc(mean(Saliency_Map,3));title('Saliency Map');
          axis off;subplot(1, 4, 4);imagesc(mean(Feature_Maps, 3));axis off;
          title('Mean Feature Map');
%%
figure,
imagesc(mean(Saliency_Map, 3))
%%

inputSize = 104;
numHiddenUnits = 100;
numClasses = 2;

layers = [ ...
    sequenceInputLayer(inputSize)
    bilstmLayer(numHiddenUnits,'OutputMode','last')
    fullyConnectedLayer(numClasses)
    softmaxLayer
    classificationLayer]
%%
maxEpochs = 20;
miniBatchSize = 8;

options = trainingOptions('adam', ...
    'ExecutionEnvironment','cpu', ...
    'GradientThreshold',1, ...
    'MaxEpochs',maxEpochs, ...
    'MiniBatchSize',miniBatchSize, ...
    'SequenceLength','longest', ...
    'Shuffle','never', ...
    'Verbose',0, ...
    'Plots','training-progress');
%%
YLabel=imds.Labels;
rng default    % Set the seed for reproducibility
c = cvpartition(YLabel,'HoldOut',0.3);
idxTrain = training(c,1);
idxTest = ~idxTrain;
XTrain = saliency_dense_features(idxTrain,:);
yTrain = YLabel(idxTrain);
XTest = saliency_dense_features(idxTest,:);
yTest = YLabel(idxTest);
%%

%%
 svmmL=fitcecoc(XTrain,yTrain);

YPred = predict(svmmL,XTest);
accuracy = mean(YPred == yTest)

%%
net = trainNetwork(XTrain,yTrain,layers,options);
%%
miniBatchSize = 8;
YPred = classify(net,XTest, ...
    'MiniBatchSize',miniBatchSize, ...
    'SequenceLength','longest');
%%
acc = sum(YPred == yTest)./numel(yTest)
%%
denNet=densenet201;
%%
imageSize = denNet.Layers(1).InputSize;
augmentedSet = augmentedImageDatastore(imageSize, imds, 'ColorPreprocessing', 'gray2rgb');
%%
w1 = denNet.Layers(3).Weights;

% Scale and resize the weights for visualization
w1 = mat2gray(w1);
w1 = imresize(w1,5); 

% Display a montage of network weights. There are 96 individual sets of
% weights in the first layer.
figure
montage(w1)

%%
trainingOptions()
featureLayer = 'fc1000';
deepFeatures = activations(denNet, augmentedSet, featureLayer, ...
    'MiniBatchSize', 32, 'OutputAs', 'rows');
%%
saliency_dense_features=[deepFeatures,saliency_features_mean];

%% DEMO 2
nOfSelection=80;
population=20;
itrationMax=10000;
fun=@meandata;
[Selection , SelectionValue]=psoSelection(saliency_features_mean,nOfSelection,population,itrationMax,fun);
MinimizedData=saliency_features_mean(:,Selection);
disp('Selection---->')
disp(Selection)
disp('SelectionValue---->')
disp(SelectionValue)

%%
 logitt=fitcensemble(saliency_dense_features,imds.Labels,'Method','LogitBoost','KFold',10);
ce = kfoldLoss(logitt);
accuracy=1-ce
%%
svmguasian1 = fitcsvm(saliency_dense_features,imds.Labels,'Standardize',true,'KernelFunction','RBF',...
    'KernelScale','auto','KFold',10);
ce = kfoldLoss(svmguasian1);
accuracy=1-ce
%%
knnfine = fitcknn(saliency_dense_features,imds.Labels,'NumNeighbors',1,...
    'NSMethod','exhaustive','Distance','minkowski',...
    'Standardize',1,'KFold',10);
ce = kfoldLoss(knnfine);
accuracy=1-ce
%%
knnmedium = fitcknn(saliency_dense_features,imds.Labels,'NumNeighbors',10,...
    'NSMethod','exhaustive','Distance','minkowski',...
    'Standardize',1,'KFold',10);
ce = kfoldLoss(knnmedium);
accuracy=1-ce
%%
knncoarse = fitcknn(saliency_dense_features,imds.Labels,'NumNeighbors',100,...
    'NSMethod','exhaustive','Distance','minkowski',...
    'Standardize',1,'KFold',10);
ce = kfoldLoss(knncoarse);
accuracy=1-ce
%%
bagmodel = fitcensemble(saliency_dense_features,imds.Labels,'Method','Bag','KFold',10);
ce = kfoldLoss(bagmodel);
accuracy=1-ce
%%
totalboostmodel = fitcensemble(saliency_dense_features,imds.Labels,'Method','TotalBoost','KFold',10);
ce = kfoldLoss(totalboostmodel);
accuracy=1-ce
%%
gentleboostmodel = fitcensemble(saliency_dense_features,imds.Labels,'Method','GentleBoost','KFold',10);
ce = kfoldLoss(gentleboostmodel);
accuracy=1-ce
%%
robustboostmodel = fitcensemble(saliency_dense_features,imds.Labels,'Method','RobustBoost','KFold',10);
ce = kfoldLoss(robustboostmodel);
accuracy=1-ce
%%
LPboostmodel = fitcensemble(saliency_dense_features,imds.Labels,'Method','LPBoost','KFold',10);
ce = kfoldLoss(LPboostmodel);
accuracy=1-ce
%%
RUSboostmodel = fitcensemble(saliency_dense_features,imds.Labels,'Method','RUSBoost','KFold',10);
ce = kfoldLoss(RUSboostmodel);
accuracy=1-ce
%%
SubSpacemodel = fitcensemble(saliency_dense_features,imds.Labels,'Method','Subspace','KFold',10);
ce = kfoldLoss(SubSpacemodel);
accuracy=1-ce
%%
[trainZ,testZ] = ccaFuse(XTrain(:,1:1000), XTrain(:,1001:1104), XTest(:,1:1000), XTest(:,1001:1104), 'concat');
%%
 svmmL=fitcecoc(trainZ,yTrain);

YPred = predict(svmmL,testZ);
accuracy = mean(YPred == yTest)
%%
logitt=fitcensemble(trainZ,yTrain,'Method','LogitBoost');
YPred = predict(logitt,testZ);
accuracy = mean(YPred == yTest)
%%
n=grp2idx(imds.Labels);
saliency_dense_mat=[double(saliency_dense_features),n];
%%
only_saliency_mat=[saliency_features_mean,n];
%% 
only_dense_mat=[deepFeatures,n];